@@ -47,6 +47,16 @@ typedef enum {
   RELAY_METRICS_RELAY_FLAGS,
   /** Numer of circuits. */
   RELAY_METRICS_NUM_CIRCUITS,
+  /** Timestamp at which the current online keys will expire. */
+  RELAY_METRICS_SIGNING_CERT_EXPIRY,
+  /** Number of times we received an EST_REND cell */
+  RELAY_METRICS_NUM_EST_REND,
+  /** Number of times we received an EST_INTRO cell */
+  RELAY_METRICS_NUM_EST_INTRO,
+  /** Number of times we received an INTRO1 cell */
+  RELAY_METRICS_NUM_INTRO1_CELLS,
+  /** Number of times we received a REND1 cell */
+  RELAY_METRICS_NUM_REND1_CELLS,
 } relay_metrics_key_t;
 
 /** The metadata of a relay metric. */
@@ -70,4 +80,54 @@ void relay_metrics_free(void);
 /* Accessors. */
 const smartlist_t *relay_metrics_get_stores(void);
 
+typedef enum {
+  EST_INTRO_SUCCESS,
+  EST_INTRO_MALFORMED,
+  EST_INTRO_UNSUITABLE_CIRCUIT,
+  EST_INTRO_CIRCUIT_DEAD,
+
+  EST_INTRO_ACTION_COUNT
+} est_intro_action_t;
+
+void relay_increment_est_intro_action(est_intro_action_t);
+
+typedef enum {
+  EST_REND_SUCCESS,
+  EST_REND_UNSUITABLE_CIRCUIT,
+  EST_REND_SINGLE_HOP,
+  EST_REND_MALFORMED,
+  EST_REND_DUPLICATE_COOKIE,
+  EST_REND_CIRCUIT_DEAD,
+
+  EST_REND_ACTION_COUNT
+} est_rend_action_t;
+
+void relay_increment_est_rend_action(est_rend_action_t);
+
+typedef enum {
+  INTRO1_SUCCESS,
+  INTRO1_CIRCUIT_DEAD,
+  INTRO1_MALFORMED,
+  INTRO1_UNKNOWN_SERVICE,
+  INTRO1_RATE_LIMITED,
+  INTRO1_CIRCUIT_REUSED,
+  INTRO1_SINGLE_HOP,
+
+  INTRO1_ACTION_COUNT
+} intro1_action_t;
+
+void relay_increment_intro1_action(intro1_action_t);
+
+typedef enum {
+  REND1_SUCCESS,
+  REND1_UNSUITABLE_CIRCUIT,
+  REND1_MALFORMED,
+  REND1_UNKNOWN_COOKIE,
+  REND1_CIRCUIT_DEAD,
+
+  REND1_ACTION_COUNT
+} rend1_action_t;
+
+void relay_increment_rend1_action(rend1_action_t);
+
 #endif /* !defined(TOR_FEATURE_RELAY_RELAY_METRICS_H) */
